@@ -21,6 +21,8 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 #ifndef _BGP_TABLE_H_
 #define _BGP_TABLE_H_
 
+#define DEFAULT_BGP_INFO_HASH 13
+
 /* AFI and SAFI type. */
 typedef u_int16_t afi_t;
 typedef u_int8_t safi_t;
@@ -57,7 +59,7 @@ struct bgp_node
 #define l_left   link[0]
 #define l_right  link[1]
 
-  void *info;
+  void **info;
 
   unsigned int lock;
 /*
@@ -80,34 +82,6 @@ struct bgp_info
   struct bgp_info *prev;
   struct bgp_peer *peer;
   struct bgp_attr *attr;
-  time_t uptime;
-  unsigned int lock;
-
-  /* BGP information status.  */
-/*
-  u_int16_t flags;
-#define BGP_INFO_IGP_CHANGED    (1 << 0)
-#define BGP_INFO_DAMPED         (1 << 1)
-#define BGP_INFO_HISTORY        (1 << 2)
-#define BGP_INFO_SELECTED       (1 << 3)
-#define BGP_INFO_VALID          (1 << 4)
-#define BGP_INFO_ATTR_CHANGED   (1 << 5)
-#define BGP_INFO_DMED_CHECK     (1 << 6)
-#define BGP_INFO_DMED_SELECTED  (1 << 7)
-#define BGP_INFO_STALE          (1 << 8)
-#define BGP_INFO_REMOVED        (1 << 9)
-#define BGP_INFO_COUNTED    (1 << 10)
-*/
-
-  u_char type;
-  u_char sub_type;
-  u_char valid;
-
-  /* MPLS label.  */
-// u_char tag[3];
-
-  /* AS-Pathlimit TTL */
-  u_char ttl;
 };
 
 /* Prototypes */
@@ -123,14 +97,11 @@ EXT struct bgp_node *bgp_table_top (const struct bgp_table *const);
 EXT struct bgp_node *bgp_route_next (struct bgp_node *);
 EXT struct bgp_node *bgp_route_next_until (struct bgp_node *, struct bgp_node *);
 EXT struct bgp_node *bgp_node_get (struct bgp_table *const, struct prefix *);
-EXT struct bgp_node *bgp_node_lookup (const struct bgp_table *const, struct prefix *);
 EXT struct bgp_node *bgp_lock_node (struct bgp_node *node);
-EXT struct bgp_node *bgp_node_match (const struct bgp_table *, struct prefix *);
-EXT struct bgp_node *bgp_node_match_ipv4 (const struct bgp_table *,
-					  struct in_addr *);
+EXT struct bgp_node *bgp_node_match (const struct bgp_table *, struct prefix *, struct bgp_peer *);
+EXT struct bgp_node *bgp_node_match_ipv4 (const struct bgp_table *, struct in_addr *, struct bgp_peer *);
 #ifdef ENABLE_IPV6
-EXT struct bgp_node *bgp_node_match_ipv6 (const struct bgp_table *,
-					  struct in6_addr *);
+EXT struct bgp_node *bgp_node_match_ipv6 (const struct bgp_table *, struct in6_addr *, struct bgp_peer *);
 #endif /* ENABLE_IPV6 */
 EXT unsigned long bgp_table_count (const struct bgp_table *const);
 
