@@ -5,7 +5,7 @@
 #endif
 
 EXT u_int32_t PdataSz, ChBufHdrSz, CharPtrSz, CounterSz, HostAddrSz;
-EXT u_int32_t PpayloadSz, PextrasSz;
+EXT u_int32_t PpayloadSz, PextrasSz, PmsgSz;
 EXT u_int32_t NfHdrV5Sz, NfHdrV1Sz, NfHdrV7Sz, NfHdrV8Sz, NfHdrV9Sz;
 EXT u_int32_t NfDataHdrV9Sz, NfTplHdrV9Sz, NfOptTplHdrV9Sz;
 EXT u_int32_t NfDataV1Sz, NfDataV5Sz, NfDataV7Sz;
