@@ -1,6 +1,6 @@
 /*
     pmacct (Promiscuous mode IP Accounting package)
-    pmacct is Copyright (C) 2003-2009 by Paolo Lucente
+    pmacct is Copyright (C) 2003-2010 by Paolo Lucente
 */
 
 /*
@@ -61,7 +61,8 @@ struct chained_cache *P_cache_search(struct pkt_primitives *, struct pkt_bgp_pri
 void P_cache_insert(struct pkt_data *, struct pkt_bgp_primitives *);
 void P_cache_flush(struct chained_cache *[], int);
 void P_cache_purge(struct chained_cache *[], int);
-void P_write_stats_header();
+void P_write_stats_header_formatted();
+void P_write_stats_header_csv();
 void P_exit_now(int);
 
 /* global vars */
