@@ -105,7 +105,7 @@ void process_query_data(int sd, unsigned char *buf, int len, int forked)
     q->what_to_count = config.what_to_count; 
     for (idx = 0; idx < config.buckets; idx++) {
       if (!following_chain) acc_elem = (struct acc *) elem;
-      if (acc_elem->packet_counter && !acc_elem->reset_flag) {
+      if (acc_elem->bytes_counter && !acc_elem->reset_flag) {
 	enQueue_elem(sd, &rb, acc_elem, PdataSz, PdataSz+PbgpSz);
 	/* XXX: to be optimized ? */
 	if (PbgpSz) {
@@ -142,7 +142,7 @@ void process_query_data(int sd, unsigned char *buf, int len, int forked)
 
       do {
         if (following_chain) acc_elem = acc_elem->next;
-        if (acc_elem->packet_counter && !acc_elem->reset_flag) bd.howmany++;
+        if (acc_elem->bytes_counter && !acc_elem->reset_flag) bd.howmany++;
         bd.num = idx; /* we need to avoid this redundancy */
         following_chain = TRUE;
       } while (acc_elem->next != NULL);
@@ -162,7 +162,7 @@ void process_query_data(int sd, unsigned char *buf, int len, int forked)
       if (request.what_to_count == config.what_to_count) { 
         acc_elem = search_accounting_structure(&request.data, &request.pbgp);
         if (acc_elem) { 
-	  if (acc_elem->packet_counter && !acc_elem->reset_flag) {
+	  if (acc_elem->bytes_counter && !acc_elem->reset_flag) {
 	    enQueue_elem(sd, &rb, acc_elem, PdataSz, PdataSz+PbgpSz);
 	    /* XXX: to be optimized ? */
 	    if (PbgpSz) {
@@ -203,7 +203,7 @@ void process_query_data(int sd, unsigned char *buf, int len, int forked)
 
         for (idx = 0; idx < config.buckets; idx++) {
           if (!following_chain) acc_elem = (struct acc *) elem;
-	  if (acc_elem->packet_counter && !acc_elem->reset_flag) {
+	  if (acc_elem->bytes_counter && !acc_elem->reset_flag) {
 	    mask_elem(&tbuf, &bbuf, acc_elem, request.what_to_count); 
             if (!memcmp(&tbuf, &request.data, sizeof(struct pkt_primitives)) &&
 		!memcmp(&bbuf, &request.pbgp, sizeof(struct pkt_bgp_primitives))) {
@@ -244,7 +244,8 @@ void process_query_data(int sd, unsigned char *buf, int len, int forked)
 
     /* XXX: we should try using pmct_get_max_entries() */
     q->num = config.classifier_table_num;
-    if (!q->num && config.classifiers_path) q->num = MAX_CLASSIFIERS;
+    // if (!q->num && config.classifiers_path) q->num = MAX_CLASSIFIERS;
+    if (!q->num && class) q->num = MAX_CLASSIFIERS;
 
     while (idx < q->num) {
       enQueue_elem(sd, &rb, &class[idx], sizeof(struct stripped_class), sizeof(struct stripped_class));
@@ -273,6 +274,7 @@ void mask_elem(struct pkt_primitives *d1, struct pkt_bgp_primitives *d2, struct
   if (w & COUNT_SRC_MAC) memcpy(d1->eth_shost, s1->eth_shost, ETH_ADDR_LEN); 
   if (w & COUNT_DST_MAC) memcpy(d1->eth_dhost, s1->eth_dhost, ETH_ADDR_LEN); 
   if (w & COUNT_VLAN) d1->vlan_id = s1->vlan_id; 
+  if (w & COUNT_COS) d1->cos = s1->cos; 
 #endif
   if (w & (COUNT_SRC_HOST|COUNT_SRC_NET)) {
     if (s1->src_ip.family == AF_INET) d1->src_ip.address.ipv4.s_addr = s1->src_ip.address.ipv4.s_addr; 
